@@ -14,44 +14,43 @@ An ACME Account
 
 use base64::URL_SAFE_NO_PAD;
 
-use serde_json::json; 
+use serde_json::json;
 
-use crate::jose::{jose_req, key_authorization_sha256};
-use crate::acme::{AcmeError, ChallengeType, Directory, get_header, Order, Identifier, Auth, Challenge};
-use generic_async_http_client::{Response};
-use crate::crypto::{EcdsaP256SHA256KeyPair, sha256_hasher};
-use std::path::{PathBuf, Path};
-use crate::fs::{create_dir_all, read_if_exist, write_file};
+use crate::{
+    acme::{get_header, AcmeError, Auth, Challenge, ChallengeType, Directory, Identifier, Order},
+    cache::AcmeCache,
+    crypto::EcdsaP256SHA256KeyPair,
+    jose::{jose_req, key_authorization_sha256},
+};
+use generic_async_http_client::Response;
 
 #[derive(Debug)]
 pub struct Account {
     key_pair: EcdsaP256SHA256KeyPair,
     directory: Directory,
-    cache: Option<PathBuf>,
     kid: String,
 }
 
 impl Account {
     /// Create or load a cached Account for ACME provider at `directory`.
     /// Provide your email in `contact` in the form *mailto:admin@example.com* to receive warnings regarding your certificate.
-    /// Set a `cache_dir` to remember/load your account.
-    pub async fn load_or_create<'a, P, S, I>(
+    /// Set a `cache` to remember/load your account.
+    pub async fn load_or_create<'a, C, S, I>(
         directory: Directory,
-        cache_dir: Option<P>,
+        cache: Option<&C>,
         contact: I,
     ) -> Result<Self, AcmeError>
     where
-        P: AsRef<Path>,
+        C: AcmeCache,
         S: AsRef<str> + 'a,
         I: IntoIterator<Item = &'a S>,
     {
-        if let Some(cache_dir) = &cache_dir {
-            create_dir_all(cache_dir).await?;
-        }
         let contact: Vec<&'a str> = contact.into_iter().map(AsRef::<str>::as_ref).collect();
-        let file = Self::cached_key_file_name(&contact);
-        let pkcs8 = match &cache_dir {
-            Some(cache_dir) => read_if_exist(cache_dir, &file).await?,
+        let pkcs8 = match &cache {
+            Some(cache) => cache
+                .read_account(&contact)
+                .await
+                .map_err(AcmeError::cache)?,
             None => None,
         };
         let key_pair = match pkcs8 {
@@ -64,15 +63,24 @@ impl Account {
                 match EcdsaP256SHA256KeyPair::generate() {
                     Ok(pkcs8) => {
                         let data = pkcs8.as_ref();
-                        if let Some(cache_dir) = &cache_dir {
-                            write_file(cache_dir, &file, data).await?;
+                        if let Some(cache) = &cache {
+                            cache
+                                .write_account(&contact, data)
+                                .await
+                                .map_err(AcmeError::cache)?;
                         }
                         EcdsaP256SHA256KeyPair::load(data)
-                    },
-                    Err(_) => Err(())
+                    }
+                    Err(_) => Err(()),
                 }
             }
-        }.map_err(|_|AcmeError::Io(std::io::Error::new(std::io::ErrorKind::InvalidData, "could not create key pair")))?;
+        }
+        .map_err(|_| {
+            AcmeError::Io(std::io::Error::new(
+                std::io::ErrorKind::InvalidData,
+                "could not create key pair",
+            ))
+        })?;
         let payload = json!({
             "termsOfServiceAgreed": true,
             "contact": contact,
@@ -84,24 +92,15 @@ impl Account {
             &directory.nonce().await?,
             &directory.new_account,
             &payload,
-        ).await?;
+        )
+        .await?;
         let kid = get_header(&response, "Location")?;
         Ok(Account {
             key_pair,
             kid,
             directory,
-            cache: cache_dir.map(|p| p.as_ref().to_path_buf()),
         })
     }
-    fn cached_key_file_name(contact: &Vec<&str>) -> String {
-        let mut ctx = sha256_hasher();
-        for el in contact {
-            ctx.update(el.as_ref());
-            ctx.update(&[0])
-        }
-        let hash = base64::encode_config(ctx.finish(), base64::URL_SAFE_NO_PAD);
-        format!("cached_account_{}", hash)
-    }
     /// send a JOSE request using the own Key to sign it
     async fn request(&self, url: impl AsRef<str>, payload: &str) -> Result<Response, AcmeError> {
         jose_req(
@@ -110,11 +109,12 @@ impl Account {
             &self.directory.nonce().await?,
             url.as_ref(),
             payload,
-        ).await
+        )
+        .await
     }
     /// send a new order for the DNS identifiers in domains
     pub async fn new_order(&self, domains: Vec<String>) -> Result<Order, AcmeError> {
-        let domains: Vec<Identifier> = domains.into_iter().map(|d| Identifier::Dns(d)).collect();
+        let domains: Vec<Identifier> = domains.into_iter().map(Identifier::Dns).collect();
         let payload = format!("{{\"identifiers\":{}}}", serde_json::to_string(&domains)?);
         let mut response = self.request(&self.directory.new_order, &payload).await?;
         Ok(response.json().await?)
@@ -147,12 +147,11 @@ impl Account {
     /// the hash needs to be presented inside the TLS certificate when the ACME TLS ALPN is present
     pub fn tls_alpn_01<'a>(
         &self,
-        challenges: &'a Vec<Challenge>,
+        challenges: &'a [Challenge],
     ) -> Result<(&'a Challenge, impl AsRef<[u8]>), AcmeError> {
         let challenge = challenges
             .iter()
-            .filter(|c| c.typ == ChallengeType::TlsAlpn01)
-            .next();
+            .find(|c| c.typ == ChallengeType::TlsAlpn01);
         let challenge = match challenge {
             Some(challenge) => challenge,
             None => return Err(AcmeError::NoTlsAlpn01Challenge),
