@@ -7,10 +7,9 @@ Without anything specified you will end up with *no async backend selected* or *
 */
 
 pub mod acme;
-mod jose;
+pub mod cache;
 mod crypto;
-mod fs;
+mod jose;
 
 #[cfg(feature = "use_rustls")]
 pub mod rustls_helper;
-
