@@ -1,38 +1,66 @@
-
-use std::time::Duration;
-use std::path::Path;
-use thiserror::Error;
 use futures_util::future::try_join_all;
-
 use rustls::sign::CertifiedKey;
-use crate::crypto::{get_cert_duration_left, CertBuilder, gen_acme_cert};
-use crate::acme::{Order, Directory, Account, Auth, Identifier, AcmeError};
+use std::time::Duration;
+use thiserror::Error;
+
+use crate::{
+    acme::{Account, AcmeError, Auth, Directory, Identifier, Order},
+    cache::AcmeCache,
+    crypto::{gen_acme_cert, get_cert_duration_left, CertBuilder},
+};
 
 #[cfg(feature = "use_async_std")]
 use async_std::task::sleep;
 #[cfg(feature = "use_tokio")]
 use tokio::time::sleep;
 
-/// Request a signed certificate from the ACME provider at `directory_url` for the DNS `domains`.
+/// Obtain a signed certificate from the ACME provider at `directory_url` for the DNS `domains`.
+///
 /// The secret for the challenge is passed as a ready to use certificate to `set_auth_key(domain, certificate)?`.
 /// This certificate has to be presented upon a TLS request with ACME ALPN and SNI for that domain.
 ///
 /// Provide your email in `contact` in the form *mailto:admin@example.com* to receive warnings regarding your certificate.
-/// Set a `cache_dir` to remember your account.
-pub async fn order<P, F>(
+/// Set a `cache` to remember your account.
+pub async fn order<C, F>(
     set_auth_key: F,
     directory_url: &str,
-    domains: &Vec<String>,
-    cache_dir: Option<P>,
-    contact: &Vec<String>,
+    domains: &[String],
+    cache: Option<&C>,
+    contact: &[String],
 ) -> Result<CertifiedKey, OrderError>
-where P: AsRef<Path>, F: Fn(String, CertifiedKey) -> Result<(), AcmeError>
+where
+    C: AcmeCache,
+    F: Fn(String, CertifiedKey) -> Result<(), AcmeError>,
+{
+    let directory = Directory::discover(directory_url).await?;
+    let account = Account::load_or_create(directory, cache, contact).await?;
+
+    let (c, key_pem, cert_pem) = drive_order(set_auth_key, domains.to_vec(), account).await?;
+
+    if let Some(dir) = cache {
+        dir.write_certificate(domains, directory_url, &key_pem, &cert_pem)
+            .await
+            .map_err(AcmeError::cache)?;
+    };
+    Ok(c)
+}
+
+/// Obtain a signed certificate for the DNS `domains` using `account`.
+///
+/// The secret for the challenge is passed as a ready to use certificate to `set_auth_key(domain, certificate)?`.
+/// This certificate has to be presented upon a TLS request with ACME ALPN and SNI for that domain.
+///
+/// Returns the signed Certificate, its private key as pem, and the certificate as pem again
+pub async fn drive_order<F>(
+    set_auth_key: F,
+    domains: Vec<String>,
+    account: Account,
+) -> Result<(CertifiedKey, String, String), OrderError>
+where
+    F: Fn(String, CertifiedKey) -> Result<(), AcmeError>,
 {
     let cert = CertBuilder::gen_new(domains.clone())?;
-    
-    let directory = Directory::discover(&directory_url).await?;
-    let account = Account::load_or_create(directory, cache_dir, contact).await?;
-    let mut order = account.new_order(domains.clone()).await?;
+    let mut order = account.new_order(domains).await?;
     loop {
         order = match order {
             Order::Pending {
@@ -55,20 +83,23 @@ where P: AsRef<Path>, F: Fn(String, CertifiedKey) -> Result<(), AcmeError>
                 log::info!("download certificate");
                 let acme_cert_pem = account.obtain_certificate(certificate).await?;
                 let rd = acme_cert_pem.as_bytes();
-                let cert_key = cert.sign(rd)
-                    .map_err(|_|AcmeError::Io(std::io::Error::new(std::io::ErrorKind::InvalidData, "could not parse certificate")))?;
-                return Ok(cert_key);
+                let pkey_pem = cert.private_key_as_pem_pkcs8();
+                let cert_key = cert.sign(rd).map_err(|_| {
+                    AcmeError::Io(std::io::Error::new(
+                        std::io::ErrorKind::InvalidData,
+                        "could not parse certificate",
+                    ))
+                })?;
+                return Ok((cert_key, pkey_pem, acme_cert_pem));
             }
             Order::Invalid => return Err(OrderError::BadOrder(order)),
         }
     }
 }
-async fn authorize<F>(
-    set_auth_key: &F,
-    account: &Account,
-    url: &String) -> Result<(), OrderError>
- where F: Fn(String, CertifiedKey) -> Result<(), AcmeError>
-    {
+async fn authorize<F>(set_auth_key: &F, account: &Account, url: &str) -> Result<(), OrderError>
+where
+    F: Fn(String, CertifiedKey) -> Result<(), AcmeError>,
+{
     let (domain, challenge_url) = match account.check_auth(url).await? {
         Auth::Pending {
             identifier,
@@ -105,7 +136,7 @@ pub fn duration_until_renewal_attempt(cert_key: Option<&CertifiedKey>, err_cnt:
         .and_then(|cert_key| cert_key.cert.first())
         .and_then(|cert| get_cert_duration_left(cert.0.as_slice()).ok())
         .unwrap_or_default();
-    
+
     let wait_secs = valid_until / 2;
     match err_cnt {
         0 => wait_secs,
